@@ -29,7 +29,7 @@ use rustyline::{self, error::ReadlineError};
 use crate::EnvironmentSyncer;
 use nu_errors::ShellError;
 use nu_parser::ParserScope;
-use nu_protocol::{UntaggedValue, Value};
+use nu_protocol::{hir::ExternalRedirection, UntaggedValue, Value};
 
 use std::error::Error;
 use std::iter::Iterator;
@@ -48,16 +48,14 @@ pub fn search_paths() -> Vec<std::path::PathBuf> {
     }
 
     if let Ok(config) = nu_data::config::config(Tag::unknown()) {
-        if let Some(plugin_dirs) = config.get("plugin_dirs") {
-            if let Value {
-                value: UntaggedValue::Table(pipelines),
-                ..
-            } = plugin_dirs
-            {
-                for pipeline in pipelines {
-                    if let Ok(plugin_dir) = pipeline.as_string() {
-                        search_paths.push(PathBuf::from(plugin_dir));
-                    }
+        if let Some(Value {
+            value: UntaggedValue::Table(pipelines),
+            ..
+        }) = config.get("plugin_dirs")
+        {
+            for pipeline in pipelines {
+                if let Ok(plugin_dir) = pipeline.as_string() {
+                    search_paths.push(PathBuf::from(plugin_dir));
                 }
             }
         }
@@ -160,7 +158,9 @@ pub async fn cli(mut context: EvaluationContext) -> Result<(), Box<dyn Error>> {
                 let prompt_line = prompt.as_string()?;
 
                 context.scope.enter_scope();
-                let (prompt_block, err) = nu_parser::parse(&prompt_line, 0, &context.scope);
+                let (mut prompt_block, err) = nu_parser::parse(&prompt_line, 0, &context.scope);
+
+                prompt_block.set_redirect(ExternalRedirection::Stdout);
 
                 if err.is_some() {
                     context.scope.exit_scope();
@@ -412,7 +412,7 @@ mod tests {
     #[quickcheck]
     fn quickcheck_parse(data: String) -> bool {
         let (tokens, err) = nu_parser::lex(&data, 0);
-        let (lite_block, err2) = nu_parser::block(tokens);
+        let (lite_block, err2) = nu_parser::parse_block(tokens);
         if err.is_none() && err2.is_none() {
             let context = basic_evaluation_context().unwrap();
             let _ = nu_parser::classify_block(&lite_block, &context.scope);
